import math
import Image,ImageDraw


def scanImage(fn):
    im=Image.open(fn)

    lines=[]
    while True:
        p=findBlackPixel(im)
        if p:
            print p
            newLine=scanLine(im,p)

            if newLine==None:
                erasePixel(im, p)
            else:
                eraseLine(im,newLine)
                #print newLine
                lines.append(newLine)
            im.save("foo.png")
        else:
            return lines

def eraseLine(im,line):
    draw=ImageDraw.Draw(im)
    for segment in line:
        draw.line(segment, fill=(0,0,0,0),width=8)
    del draw

def erasePixel(im, p):
    draw=ImageDraw.Draw(im)

    px,py=p
    draw.line((px,py,px+1,py+1), fill=(0,0,0,0),width=8)
    del draw

    

def findBlackPixel(im):
    imX,imY=im.size

    for y in range(imY):
        for x in range(imX):
            p=im.getpixel((x,y))

            if p[3]>192:
                return x,y
    return None

STEP_SIZE=10
NUM_TRIES=6

MAX_LEN=40

def scanLine(im, p):

    for i in range(NUM_TRIES):
        initAngle=i*2*math.pi/NUM_TRIES
    
        a=scanOutFromAngle(im, p, initAngle, STEP_SIZE)

        if not (a==None):
            return scanOneWay(im, p, a, STEP_SIZE, MAX_LEN)+scanOneWay(im, p, a+math.pi, STEP_SIZE, MAX_LEN)
    return None


def offset(p, a, STEP_SIZE):
    px,py=p
    dx=int(math.cos(a)*STEP_SIZE)
    dy=int(math.sin(a)*STEP_SIZE)
    return px+dx, py+dy


def scanOneWay(im, p, a, step, remainingLength):
    if remainingLength<=0:
        return []
    newAngle=scanOutFromAngle(im, p, a, step)
    if newAngle is None:
        return []
    newPoint=offset(p,newAngle,step)
    retVal=[(p,newPoint)]+scanOneWay(im, newPoint, newAngle, step, remainingLength-1)
    #print "s1w returning",retVal
    return retVal

def scanOutFromAngle(im, p, initAngle, step):
    incAngle=0.5/step

    angleDelta=0

    while angleDelta<math.pi/2.0:
        newX,newY=offset(p,initAngle+angleDelta, step)
        try:
            pixel=im.getpixel((newX,newY))
            if pixel[3]>192:
                return initAngle+angleDelta
        except IndexError:
            pass
        
        newX,newY=offset(p,initAngle-angleDelta, step)
        try:
            pixel=im.getpixel((newX,newY))
            if pixel[3]>192:
                return initAngle-angleDelta
        except IndexError:
            pass

        angleDelta+=incAngle
    return None
        
    
    
    
    


lines=scanImage("../Reference/logo.png")

newIm=Image.new("RGB",(1000,1000))
d=ImageDraw.Draw(newIm)
for line in lines:
    for segment in line:
        #print segment
        d.line(segment)
del d
newIm.save("sketchLogo.png")
