import fsmMgr

import math
import copy

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import screenInfo
import mouseInfo
import statePower

class StateEdit(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.selection=None
        pygame.mouse.set_pos((screenInfo.gWindowX/2,
                              screenInfo.gWindowY/2))
        self.clicked=False

        self.lastCreatedNum=0
        
    def update(self, dt):
        mouseInfo.readMouse()
        mx=mouseInfo.gMouseX
        my=mouseInfo.gMouseY

        if self.selection:
            self.selection["center"]=(mx,my)

    def handleMouseButton(self, bDown, buttonIndex):
        if bDown:
            if buttonIndex==1:
                if self.selection:
                    # drop whatever you're holding
                    self.selection=None
                else:
                    self.selection=self.getClosestObj()
            else:
                print 'objs[99]=['
                for p in self.parent.parent.physicsObjects:
                    print '    {"name":"%s",'%p["name"]
                    print '     "center":%s,'%str(p["center"])
                    print '     "radius":%d,'%p["radius"]
                    print '     "mass":%.2f,'%p["mass"]
                    if p.get("target",False):
                        print '     "target": True,'
                    print '     "color":(%.2f, %.2f, %.2f)},'%p["color"]
                    print
                print ']'
                    
                fsmMgr.popState()
        return True

    def handleKey(self, key):
        if key==K_c:
            if self.selection:
                newObj=copy.copy(self.selection)
                newObj["name"]="newPlanet%04d"%self.lastCreatedNum
                self.lastCreatedNum+=1
                self.parent.parent.physicsObjects.append(newObj)
                return True
        if key==K_x:
            if self.selection:
                self.parent.parent.physicsObjects.remove(self.selection)
                self.selection=None
                
        return False
            
                    
    def getClosestObj(self):
        mx=mouseInfo.gMouseX
        my=mouseInfo.gMouseY

        bestDist=2000
        bestObj=None
        for p in self.parent.parent.physicsObjects:
            cx,cy=p["center"]
            dx=cx-mx
            dy=cy-my
            rSqr=dx*dx+dy*dy
            r=math.sqrt(rSqr)
            if r<bestDist:
                bestObj=p
                bestDist=r
        return bestObj
        


    def draw(self):
        # first, let our parent draw the map
        self.parent.draw()
        
        glBegin(GL_LINES)

        cursor_radius=20
        glColor3f(0, 0.7, 1)
        glVertex2f(mouseInfo.gMouseX-cursor_radius,
                   mouseInfo.gMouseY)
        glVertex2f(mouseInfo.gMouseX+cursor_radius,
                   mouseInfo.gMouseY)
        glVertex2f(mouseInfo.gMouseX,
                   mouseInfo.gMouseY-cursor_radius)
        glVertex2f(mouseInfo.gMouseX,
                   mouseInfo.gMouseY+cursor_radius)
                   

        glEnd()
