import math
import sys

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr
import mouseInfo

import statePlayAd
import stateDialog
import level1
import stateAim
import edgesMenu
import widgets
import tutorialProgress
import playerMgr
import stateNewGameMenu       
import stateOptions
import soundMgr

#seconds to scan the height of the screen
SCAN_SPEED=1.6

SCAN_INTERVAL=4

BRIGHT_COLOR=(0.7, 1, 0.7)
DIM_COLOR=(0.0, 0.55, 0.0)
FADE_TIME=0.4

SHOW_DELAY=2.0


def playTutorial():
    tutorialProgress.startTutorial()
    playerMgr.initPlayers(1,0)
    level1.thisLevel.setLevel(1)
    
    fsmMgr.pushState(level1.thisLevel)
    fsmMgr.pushState(stateAim.StateAim(None))


    if tutorialProgress.gTutorial:
        fsmMgr.pushState(stateDialog.StateDialog(("Memo from Mr. Tigerbaum","""
You will be responsible for sending
packages to the planets marked with
an X.

Use the blue cursor to aim the
PACKAGE SLINGSHOT and click the left
mouse button.
""")))
    
        fsmMgr.pushState(stateDialog.StateDialog(("Memo from Mr. Tigerbaum","""
Attention New Employee:



Welcome to
Woody Tigerbaum's interplanetary
package delivery.

It is now time to begin your training.
You will be instructed in using our
patented PACKAGE SLINGSHOT.
""")))




    
    pass

def playGame():
    fsmMgr.popAll()
    fsmMgr.pushState(stateNewGameMenu.StateNewGameMenu(None))


def watchAd():
    fsmMgr.pushState(statePlayAd.StatePlayAd(None))

def optionsMenu():
    fsmMgr.popAll()
    fsmMgr.pushState(stateOptions.StateOptions(None))
    

class StateMainMenu(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.angle=0 #east, I guess

        self.clicked=False

        #genEdgeList()
        self.mRasterPos=600
        self.splashTime=0.0
        self.rasterSpeed=600.0/SCAN_SPEED

        self.widgets=[widgets.Button(40,220,8,"Play Tutorial",playTutorial),
                      widgets.Button(40,170,8,"Play",playGame),
                      widgets.Button(40,120,8,"Intro Movie",watchAd),
                      widgets.Button(40,70,8,"Options",optionsMenu),
                      widgets.Button(40,20,8,"Quit",lambda:fsmMgr.popAll())
                      ]

        maxWidth=max(map(lambda x:x.width,self.widgets))
        for w in self.widgets:
            w.setWidth(maxWidth)



    def update(self, dt):

        dt=dt/1000.0
        self.splashTime+=dt

        self.mRasterPos-=self.rasterSpeed*dt

        mouseInfo.readMouse()
        for w in self.widgets:
            w.update(dt)

    def draw(self):
        for i in edgesMenu.lines.keys():
            if i<self.mRasterPos:
                continue

            distFromRaster=i-self.mRasterPos
            secondsSinceRaster=distFromRaster/self.rasterSpeed

            if secondsSinceRaster>FADE_TIME:
                glColor3f(*DIM_COLOR)
            else:
                br,bg,bb=BRIGHT_COLOR
                dr,dg,db=DIM_COLOR

                fracSinceRaster=secondsSinceRaster/FADE_TIME

                interpR=(dr-br)*fracSinceRaster+br
                interpG=(dg-bg)*fracSinceRaster+bg
                interpB=(db-bb)*fracSinceRaster+bb
                
                glColor3f(interpR, interpG, interpB)
                
                
            glBegin(GL_LINES)


            
            crossingList=edgesMenu.lines[i][:]
            
            while crossingList:
                end1=crossingList.pop()
                end2=crossingList.pop()
                glVertex(end1,i)
                glVertex(end2,i)

            glEnd()
            
        for w in self.widgets:
            w.draw()

    def handleMouseButton(self, bDown, buttonIndex):
        if bDown and buttonIndex==1:
            for w in self.widgets:
                if w.isMouseInMe():
                    soundMgr.playSound("ping")
                    w.callbackFunc()
        return True

    def done(self):
        fsmMgr.popState()

    

