import math

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import fsmMgr

import textDraw

import stateTitleSplash

import edgesWoody
import edgesLogo
import edgesTower
import edgesPackage
import edgesUp
import edgesLogo

slideLines={"Woody": edgesWoody.lines,
            "Logo" : edgesLogo.lines,
            "Tower": edgesTower.lines,
            "Package": edgesPackage.lines,
            "Up"   : edgesUp.lines}

#seconds to scan the height of the screen
SCAN_SPEED=1.6

SCAN_INTERVAL=4

BRIGHT_COLOR=(0.7, 1, 0.7)
DIM_COLOR=(0.0, 0.75, 0.0)
FADE_TIME=0.4

SHOW_DELAY=2.0

TITLE_DELAY=2.0


SUBTITLE_SIZE = 5

START_FADE_BEFORE_NEXT_SLIDE=2.0
END_FADE_BEFORE_NEXT_SLIDE=1.0

screens=[(    0, "Woody", False),
         ( 5278, "Logo", False),
         (12574, "Tower", False),
         (18500, "Package", False),
         (24426, "Up", False),
         (27867, "Logo", False)]

titles=[(14385, "do not try this at home", True),
        (20543, "Not a guarantee", True),
        (23240, "The only way to go is up", True),
        (28449, "Offer not valid in delaware", True)]

class StatePlayAd(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)
        self.angle=0 #east, I guess

        self.clicked=False

        #genEdgeList()
        self.mRasterPos1=600
        self.mRasterPos2=600
        self.splashTime=0.0
        self.rasterSpeed=600.0/SCAN_SPEED

        self.showDelay=False
        self.showDelayTime=0.0

        pygame.mixer.music.load("Data/intro1.ogg")
        pygame.mixer.music.play()

        

    def update(self, dt):
        dt=dt/1000.0
        self.splashTime+=dt

        
        self.mRasterPos1-=self.rasterSpeed*dt
        if self.splashTime>SCAN_SPEED*0.5:
            self.mRasterPos2-=self.rasterSpeed*dt

        if ((not self.showDelay) and
            (not pygame.mixer.music.get_busy())):
            self.showDelay=True

        if self.showDelay:
            self.showDelayTime+=dt
            if self.showDelayTime>SHOW_DELAY:
                self.done()

    def draw(self):
        soundTime=pygame.mixer.music.get_pos()

        self.drawTitles(soundTime)
        self.drawSlides(soundTime)


    def computeColor(self, initTime, timeNow):
        elapsedTime=timeNow-initTime
        
        if elapsedTime>FADE_TIME:
            return DIM_COLOR
        else:
            br,bg,bb=BRIGHT_COLOR
            dr,dg,db=DIM_COLOR

            fracSinceRaster=elapsedTime/FADE_TIME

            interpR=(dr-br)*fracSinceRaster+br
            interpG=(dg-bg)*fracSinceRaster+bg
            interpB=(db-bb)*fracSinceRaster+bb

            return (interpR, interpG, interpB)
                
                
        
    def drawTitles(self, soundTime):
        for s in titles:
            tm, titleMsg, bogus=s
            if tm>soundTime:
                continue
            delta=(soundTime-tm)/1000.0
            if delta>TITLE_DELAY:
                continue
            textColor=self.computeColor(tm/1000.0, soundTime/1000.0)

            w=textDraw.stringWidth(SUBTITLE_SIZE, titleMsg)

            textDraw.drawString(400-w/2,
                                2*SUBTITLE_SIZE,
                                textColor,
                                SUBTITLE_SIZE,
                                titleMsg)

    def drawSlides(self,soundTime):
        soundTimeInSeconds=soundTime/1000.0
        self.drawSlide(0.0,
                       screens[0][1],
                       soundTimeInSeconds,
                       screens[1][0]/1000.0)

        for i,slide in enumerate(screens):
            if i==0:
                continue
            
            tm, slideName, bogus=slide
            
            startTime=tm/1000.0-SCAN_SPEED
            if i+1==len(screens):
                endTime=10000.0
            else:
                endTime=screens[i+1][0]/1000.0
            self.drawSlide(startTime,
                           slideName,
                           soundTimeInSeconds,
                           endTime)
                           

    def drawSlide(self,
                  startTime,
                  screenName,
                  curTime,
                  nextSlideTime):
        if ((curTime<startTime) or
            (curTime>nextSlideTime-END_FADE_BEFORE_NEXT_SLIDE)):
            return

        timeSinceStart=curTime-startTime
        rasterPos=600-(600.0*timeSinceStart/SCAN_SPEED)
        fadeFrac=1.0
        if curTime>nextSlideTime-START_FADE_BEFORE_NEXT_SLIDE:
            fadeFrac=1.0-((curTime+START_FADE_BEFORE_NEXT_SLIDE-nextSlideTime)/
                          (START_FADE_BEFORE_NEXT_SLIDE-END_FADE_BEFORE_NEXT_SLIDE))

        lines=slideLines[screenName]

        for i in lines.keys():
            if i<rasterPos:
                continue

            distFromRaster=i-rasterPos
            secondsSinceRaster=distFromRaster/self.rasterSpeed

            lcR,lcG,lcB=self.computeColor(0,secondsSinceRaster)
            glColor3f(lcR*fadeFrac,lcG*fadeFrac,lcB*fadeFrac)
                
            glBegin(GL_LINES)
            
            crossingList=lines[i][:]
            
            while crossingList:
                end1=crossingList.pop()
                end2=crossingList.pop()
                glVertex(end1,i)
                glVertex(end2,i)

            glEnd()


    def drawSecondPass(self):
        for i in edgesPW4_2.lines.keys():
            if i<self.mRasterPos2:
                continue

            distFromRaster=i-self.mRasterPos2
            secondsSinceRaster=distFromRaster/self.rasterSpeed

            if secondsSinceRaster>FADE_TIME:
                glColor3f(*DIM_COLOR)
            else:
                br,bg,bb=BRIGHT_COLOR
                dr,dg,db=DIM_COLOR

                fracSinceRaster=secondsSinceRaster/FADE_TIME

                interpR=(dr-br)*fracSinceRaster+br
                interpG=(dg-bg)*fracSinceRaster+bg
                interpB=(db-bb)*fracSinceRaster+bb
                
                glColor3f(interpR, interpG, interpB)
                
                
            glBegin(GL_LINES)
            
            crossingList=edgesPW4_2.lines[i][:]
            
            while crossingList:
                end1=crossingList.pop()
                end2=crossingList.pop()
                glVertex(end1,i)
                glVertex(end2,i)

            glEnd()



    def handleMouseButton(self, bDown, buttonIndex):
        #calibrate power next

        if bDown==True and buttonIndex==1:
            self.clicked=True
        if bDown==False and buttonIndex==1 and self.clicked:
            self.done()
        return True

    def handleKey(self, key):
        self.done()
        return True


    def done(self):
        pygame.mixer.music.fadeout(10)
        fsmMgr.popState()

