import fsmMgr

import math
import random

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *

import mouseInfo
import stateFlight
import statePower
import tutorialProgress
import playerMgr
import soundMgr

MARKER_SPEED=0.8
BASE_SPEED=300.0

BASE_ANGLE_ERR=0.7

POWER_SCALE=0.2

def genericLaunch(parent, accuracy):
    #set the package on its way
    power=parent.power
    angle=parent.angle
    curPlan=playerMgr.getCurrentPlanet()

    print "accuracy: %2.1f%%"%(accuracy*100)

    missFrac=1.0-accuracy

    angleNorm=BASE_ANGLE_ERR*missFrac
    angleErr=random.gauss(angleNorm,angleNorm)+random.gauss(-angleNorm,angleNorm)

    powerNorm=power*missFrac*POWER_SCALE
    powerErr=random.gauss(powerNorm,powerNorm)+random.gauss(-powerNorm, powerNorm)
    print "angle error : %3.1f degrees"%math.degrees(abs(angleErr))
    print "power error: %2.1f%%"%(powerErr*100.0)

    angle+=angleErr
    power+=powerErr
    power=abs(power)
    curPlanRad=curPlan["radius"]
    curPlanX,curPlanY=curPlan["center"]

    parent.packageX=curPlanX+math.cos(angle)*curPlanRad
    parent.packageY=curPlanY+math.sin(angle)*curPlanRad

    parent.packageVX=math.cos(angle)*power*BASE_SPEED
    parent.packageVY=math.sin(angle)*power*BASE_SPEED

    parent.actualAngle=angle
    parent.actualPower=power

    if accuracy>.9:
        soundMgr.playSound("launchGood")
    else:
        soundMgr.playSound("launchBad")


class StatePutt(fsmMgr.State):
    def __init__(self, data):
        fsmMgr.State.__init__(self)

        self.puttMarker=0
        self.markerMovingUp=True

    def update(self, dt):
        tutorialProgress.gShownAccuracy=True
        self.parent.update(dt)
        # move the putt marker
        scale=1.0
        if not self.markerMovingUp:
            scale=-1.0
        self.puttMarker+=dt*MARKER_SPEED*scale/1000.0

        if self.puttMarker>1:
            self.markerMovingUp=False
            self.puttMarker=1
        elif self.puttMarker<0:
            self.markerMovingUp=True
            self.puttMarker=0

        
    def handleMouseButton(self, bDown, buttonIndex):
        #trigger this at the beginning of the click

        if bDown==True:
            fsmMgr.popState()
            fsmMgr.pushState(stateFlight.StateFlight(None))
            self.launch()
            return True
        return False

    def launch(self):
        #set the package on its way
        missFrac=abs(self.puttMarker-self.parent.power)
        accuracy=1.0-missFrac

        genericLaunch(self.parent, accuracy)

    def onButtonUp(self, buttonIndex):
        #launch the package
        fsmMgr.pop()
        fsmMgr.push(stateStrength.StateStrength(self.parent))


    def draw(self):
        # first, let our parent draw the map
        self.parent.draw()
        
        # draw an arrow out of the planet in the direction of the
        # angle.
        # make sure to start it the radius of the current planet, and
        # go out from there.

        curPlan=playerMgr.getCurrentPlanet()
        
        curPlanRad=curPlan["radius"]
        curPlanX,curPlanY=curPlan["center"]

        power=self.parent.power
        angle=self.parent.angle
        
        arrowLength=100*power
        headLength=20*power
        headAngle=0.20*power

        statePower.drawPowerMeter(self.parent.power)
        drawPuttMarker(self.puttMarker)
        glBegin(GL_LINES)
        glColor3f(1,1,0)
        
        arrowBaseX=curPlanX+math.cos(angle)*curPlanRad
        arrowBaseY=curPlanY+math.sin(angle)*curPlanRad

        arrowTipX=curPlanX+math.cos(angle)*(curPlanRad+arrowLength)
        arrowTipY=curPlanY+math.sin(angle)*(curPlanRad+arrowLength)

        glVertex2f(arrowBaseX,arrowBaseY)
        glVertex2f(arrowTipX,arrowTipY)

        arrowBarbX=curPlanX+math.cos(angle+headAngle)*(curPlanRad+arrowLength-headLength)
        arrowBarbY=curPlanY+math.sin(angle+headAngle)*(curPlanRad+arrowLength-headLength)
        glVertex2f(arrowTipX,arrowTipY)
        glVertex2f(arrowBarbX,arrowBarbY)

        arrowBarbX=curPlanX+math.cos(angle-headAngle)*(curPlanRad+arrowLength-headLength)
        arrowBarbY=curPlanY+math.sin(angle-headAngle)*(curPlanRad+arrowLength-headLength)
        glVertex2f(arrowTipX,arrowTipY)
        glVertex2f(arrowBarbX,arrowBarbY)
        glEnd()

def drawPuttMarker(puttMarker):
    glBegin(GL_LINE_LOOP)
    glColor3f(1,0,0)
    puttMarkerX=750
    puttMarkerY=600*puttMarker

    glVertex2f(puttMarkerX, puttMarkerY)
    glVertex2f(puttMarkerX+10, puttMarkerY+5)
    glVertex2f(puttMarkerX+20, puttMarkerY)
    glVertex2f(puttMarkerX+10, puttMarkerY-5)
    glEnd()

    
